import json
import requests
import TAssist.TA_Graphs
import TAssist.TA_Globals

api_url = "https://api.pegasis.site/public/yrdsb_ta/getmark_v2"

class Mark:
    def __init__(self, name, data):
        if data is not None:
            # Strange fix to compensate for data being in list of size 1. idk why its like this
            data = data[0]
            self.type = name
            self.weight = data["weight"]
            self.score = data["get"]
            self.out_of = data["total"]
            self.finished = data["finished"]

            self.percent = round((self.score / self.out_of * 100), 2)
        else:
            self.type = False
            self.weight = False
            self.score = False
            self.out_of = False
            self.finished = False 
            self.percent = 0
    
    def __repr__(self):
        return f"Mark Object at {hex(id(self))}"

class Assignment:
    def __init__(self, data, course):
        self.course = course

        self.feedback = data["feedback"]
        self.name = data["name"]

        self.KU = Mark("KU", data.get("KU"))
        self.A = Mark("A", data.get("A"))
        self.T = Mark("T", data.get("T"))
        self.C = Mark("C", data.get("C"))

        self.F = Mark("F", data.get("F"))
        self.O = Mark("O", data.get("O"))

        self.marks = {
            "KU": self.KU,
            "A": self.A,
            "T": self.T,
            "C": self.C,
            "O": self.O,
            "F": self.F
        }

        exists = 0
        total = 0
        for mark in self.marks.values():
            if mark.percent: exists += 1
            total += mark.percent
        
        # Caluclate assignment average
        self.avg = total / exists

    def __repr__(self):
        return f"Assignment Object at {hex(id(self))}"

    def get_previous_assignment(self):
        work = self.course.assignments
        pos = work.index(self)
        if pos == 0: return None
        else: return work[pos-1]

    def compare_with_previous(self, cat):
        assignment = self.get_previous_assignment()
        if assignment is None: return ""
        else:
            now = self.marks[cat].percent
            before = assignment.marks[cat].percent

            if now and before:
                if now > before: return f"You scored *{now - before}% better* than your previous assignment: {assignment.name}"
                elif now < before: return f"You scored *{before - now}% worse* than your previous assignment: {assignment.name}"
                else: return ""

class Course:
    def __init__(self, data):
        self.name = data["name"]
        self.code = data["code"]
        self.start = data["start_time"]
        self.end = data["end_time"]
        self.block = data["block"]
        self.room = data["room"]
        self.weight_table = data["weight_table"]
        self.assignments = [Assignment(item, self) for item in data["assignments"]]
        self.assign_len = len(self.assignments)
        self.overall_mark = data["overall_mark"]

        if self.overall_mark is not None:
            self.overall_mark = float(self.overall_mark)

        self.emoji = TAssist.TA_Globals.course_emojis.get(self.code[:2])
        if self.emoji is None: self.emoji = '🏫'

    def __repr__(self):
        return f"Course Object at {hex(id(self))}"

    # For finding if a course identifies with some string. Compares against course block and code
    def __eq__(self, identifier):
        # Legnth check is to make sure a block number identifier wont trigger true on numbers in course code
        if (identifier.upper() in self.code) and (len(identifier) > 1): return True
        elif identifier == self.block: return True
        else: return False

    # Find if assignment exists
    def has_assignment(self, work):
        if work is not None:
            for assignment in self.assignments:
                if work.lower() in assignment.name.lower():
                    return assignment
        return False
    
    # Generates mark graph for
    def generate_mark_graph(self, output):
        graph = TAssist.TA_Graphs.mark_graph(output, self.overall_mark, self.code)
        return graph
    
    # Generates Rose chart and Table
    def generate_grade_tables(self, output):
        # Make table/graph if data present
        if self.weight_table:
            columns = ('Category', 'Weighting', 'Course Weighting', 'Student Achievement')

            rose = TAssist.TA_Graphs.grade_rose_chart(output, self.code, self.weight_table, TAssist.TA_Globals.CATEGORIES, TAssist.TA_Globals.COLORS)
            table = TAssist.TA_Graphs.grade_table_chart(output, self.code, self.weight_table, TAssist.TA_Globals.CATEGORIES, columns, TAssist.TA_Globals.COLORS)
            
            # Return graph objects
            return rose, table
        else:
            return False

    def get_trendline(self, output):
        trend = TAssist.TA_Graphs.course_trendline(output, self.assignments, self.code, TAssist.TA_Globals.CATEGORIES, TAssist.TA_Globals.COLORS)
        return trend

class Student:
    def __init__(self, username, password):
        self.username = username
        self.password = password
        self.courses = []
        self.data = None
        self.total_average = None

    def __repr__(self):
        return f"Student Object at {hex(id(self))}"

    def has_class(self, input_course):
        for course in self.courses:
            if input_course == course:
                return course
        return False

    def fetch_data(self):
        # Make POST request to API with provided credentials
        credentials = {"number": self.username, "password": self.password}
        self.data = requests.post(api_url, json=credentials)
        status = self.data.status_code

        if status == 503 or status == 500: 
            return "[500] A server error has occured. Please try again later", False, status
        elif status == 401:
            return "[401] Incorrect password/username. Please try again", False, status
        elif status == 400:
            return "[400] Api error", False, status
        
        # Prep/prettify data and initialize it
        self.data = (self.data).json()
        self.initialize_data()
        return "[200] Succesfully connected to Teach Assist!", True, status

    def read_json(self, file):
        # Load json response data
        with open (file, "r") as f:
            self.data = json.load(f)
        self.initialize_data()


    def initialize_data(self):
        for course_info in self.data:
            # Make a new course and add it to a list
            course = Course(course_info)
            self.courses.append(course)

        self.calculte_average()

    # Calculate average from all courses
    def calculte_average(self):
        self.total_average = 0 
        num_courses = 0
        for course in self.courses:
            if course.overall_mark is not None:
                self.total_average += course.overall_mark
                num_courses += 1
        self.total_average /= num_courses


# Wrapper for graph generation code
import numpy as np
import numpy.ma as ma
import matplotlib.pyplot as plt
import matplotlib.font_manager
import matplotlib.patheffects as pe

import math

class TA_Graph:
    def __init__(self, plot, output, delimeter, name, type):
        self.output = output
        self.delimeter = delimeter
        self.name = name
        self.type =  type
        self.plot = plot

        self.filename = f'{self.delimeter}_{self.name}.{self.type}'
        self.directory = f'{output}/{self.filename}'

        self.plot.savefig(self.directory, bbox_inches='tight')

def mark_graph(output, mark, course_code):
    plt.figure().clear()
    # Prettify mark given
    if mark is None:
        mark = 0
        mark_display = '-NA-'
    else:
        mark = float(mark)
        mark = round(mark, 1)
        mark_display = str(mark) + '%'

    # Graph size
    fig, ax = plt.subplots(figsize=(6, 6), subplot_kw={'projection':'polar'})
    plt.ylim(-3, 3)

    # Set values
    bar_percentage = (mark * math.pi *2)/ 100
    bar_value = ((90 * math.pi *2)/ 360)
    bar_height = 1
    bar_color = '#00ff62'

    # Make Bars
    ax.barh(1, bar_percentage, bar_height, bar_value, color=bar_color) 
    plt.text(0, -3, mark_display, ha='center', va='center', fontsize=42)

    # Clear tick marks
    plt.xticks([])
    plt.yticks([])
    ax.spines.clear()

    # Save the graph
    return TA_Graph(fig, output, 'ring', course_code, 'png')

def prep_grade_data(data, categories, suffix=""):
    # Add a suffix to data if provided
    ad_fix = lambda num: (str(num) + suffix) if suffix else (num) 

    # Prepare lists to be populated
    weighting = []
    course_weighting = []
    student_achievement = []

    # Bring data from weight_table into lists
    for row in categories.values():
        # Get and round and store data
        weight_data = ad_fix(round(data[row]['W'], 2))
        weighting.append(weight_data)

        course_weight_data = ad_fix(round(data[row]['CW'], 2))
        course_weighting.append(course_weight_data)

        student_achievement_data = ad_fix(round(data[row]['SA'], 2))
        student_achievement.append(student_achievement_data)
    
    return weighting, course_weighting, student_achievement

def grade_rose_chart(output, course_code, data, categories, colors):
        plt.figure().clear()
        weighting, course_weighting, student_achievement = prep_grade_data(data, categories)

        # Merge info from weight table for the polar chart
        polar_angles = [0]
        polar_positions = [0]

        # Tweak data to suit the graph
        for angle in course_weighting:
            # Rescale angle data to fit graph
            rescaled_angle = math.radians(angle / 100 * 360)
            polar_angles.append(rescaled_angle)

            # Find and set position for next slice
            new_slice_position = polar_positions[-1] + ((polar_angles[-1] / 2) + (polar_angles[-2] / 2))
            polar_positions.append(new_slice_position)

        # Covert to np arrays
        polar_radii = np.array(student_achievement)
        polar_positions = np.array(polar_positions[1:])
        polar_angles = np.array(polar_angles[1:])

        # Make polar chart
        polar = plt.subplot(projection='polar')
        polar_ax = polar.bar(polar_positions, polar_radii, polar_angles, color=colors, edgecolor='black', bottom=0.0, alpha=0.5)

        # Add chart labels
        for count, bar in enumerate(polar_ax):
            if bar.get_height() > 0:
                text = list(categories.keys())[count]
                polar.text(polar_positions[count], 50, text, ha='center', va='bottom')

        # Hide axes ticks
        polar.grid(False)
        polar.set_xticks([])
        polar.set_yticks([])

        # Save the polar
        return TA_Graph(plt, output, 'polar', course_code, 'png')


def grade_table_chart(output, course_code, data, categories, columns, colors):
    plt.figure().clear()
    # Merge info from weight table into table text with '%sign' at the end
    weighting, course_weighting, student_achievement = prep_grade_data(data, categories, suffix="%")
    table_text = list(zip(list(categories.keys()), weighting, course_weighting, student_achievement))

    # Make table
    table = plt.subplot()
    table_ax = table.table(cellText=table_text,
                        colLabels=columns,
                        cellColours=[[color for n in range(len(columns))] for color in colors],
                        colLoc='center',
                        loc='top',
                        bbox=[0, 0.2, 1.2, 0.6]
                        )

    # Make first row bolded
    for (row, col), cell in table_ax.get_celld().items():
        if (row == 0):
            cell.set_text_props(fontproperties=matplotlib.font_manager.FontProperties(weight='bold'))

    # Table font size
    table_ax.set_fontsize(14)
    table_ax.scale(1.5, 1.5)

    # Hide axis
    plt.tight_layout()
    plt.axis('off')

    # Save the table
    return TA_Graph(plt, output, 'table', course_code, 'png')

def course_trendline(output, assignments, course_code, categories, color):
    plt.figure().clear()
    # line thickness
    lw = 3
    outline = lw + 1

    # x-axis
    x_axis = np.array([i for i in range(len(assignments))])

    # Plot each mark for each category
    for count, category in enumerate(categories.values()):
        info =[]

        # Dont add line if all the data in a trendline is missing
        empty_count = 0
        for assignment in assignments:
            # Mask value if non existant
            mark = assignment.marks[category].percent

            if assignment.marks[category].percent:
                info.append(mark)
            else:
                info.append(ma.masked)
                empty_count += 1
        
        if empty_count != len(assignments):
            data = ma.array(info)
            # Plot data with coresponding color, width and thin black outline
            plt.plot(x_axis, data, c = color[count], linewidth=str(lw),path_effects=[pe.Stroke(linewidth=outline, foreground='k'), pe.Normal()])

    # Set y axis scale to 0-100 and remove x-ticks
    plt.ylim((0,100))
    plt.tick_params(axis='x', which='both', bottom=False, top=False, labelbottom=False) 

    plt.title(f"Analysis/Trends for {course_code}")
    plt.xlabel("Assignments")
    plt.ylabel("Percent (%)")

    plt.grid(True)

    # Save the table
    return TA_Graph(plt, output, 'trend', course_code, 'png')

def assignment_bars(output, assignment, categories, colors):
    plt.figure().clear()
    plt.figure(figsize=(8, 6))

    # Disable axes from rendering
    ax1 = plt.axes(frameon=False)
    ax1.get_xaxis().tick_bottom()
    ax1.axes.get_yaxis().set_visible(False)
    ax1.set_ylim([0, 100])

    # creating the dataset
    values = [assignment.KU.percent, assignment.T.percent, assignment.C.percent, assignment.A.percent]
    colors = colors[:len(values)]
    category = list(categories.values())[:len(values)]
    
    # creating the bar plot
    bar = plt.bar(category, values, color = colors, width = 0.4, edgecolor="black")

    # Add text above bar
    for count, rect in enumerate(bar):
        height = rect.get_height()
        plt.text(rect.get_x() + rect.get_width() / 2.0, height, (str(values[count]) + "%" if values[count] > 0 else "-NA-"), ha='center', va='bottom')
    
    plt.xlabel("Categories")
    plt.title(f"Marks for {assignment.name} ({assignment.course.code})", pad=20)
    return TA_Graph(plt, output, 'marks', assignment.name, 'png') 


import TAssist.TA_Graphs
from TAssist.TA_Globals import CATEGORIES, COLORS, course_emojis
from TAssist.TAssist import Student
from TAssist.TAssist import Course
from TAssist.TAssist import Assignment
from TAssist.TAssist import Mark


from setuptools import setup, find_packages
import codecs
import os

VERSION = '0.0.4'
DESCRIPTION = 'An unnoficial python wrapper for the TA Public API (2.0) by Pegasis'

# Setting up
setup(
    name="TAssist",
    version=VERSION,
    author="BearGaming123 (Supreeth Govindaraju)",
    author_email="<supreethG@gmx.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=['numpy', 'matplotlib', 'requests'],
    keywords=['python', 'Teach Assist', 'YRDSB'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Microsoft :: Windows",
    ]
)